'use strict';

var React = require('react');
var reactDropzone = require('react-dropzone');
var uuid = require('uuid');
var types = require('../components/Uploader/types.js');

const useFileUpload = ({
  accept = {},
  onAccept,
  onFileRejected,
  maxSize,
  allowMultiple = true,
  preventDuplicates = false,
  onDuplicate
}) => {
  const [files, setFiles] = React.useState([]);
  const [duplicates, setDuplicates] = React.useState([]);
  const [isOpen, setIsOpen] = React.useState(false);
  const onOpen = React.useCallback(() => setIsOpen(true), []);
  const onClose = React.useCallback(() => setIsOpen(false), []);
  const updateFiles = React.useCallback(
    (prevFiles, newFiles) => {
      const current = /* @__PURE__ */ new Map();
      [...prevFiles, ...newFiles].forEach((file) => current.set(file.uid, file));
      return [...current.values()];
    },
    []
  );
  const onDrop = async (acceptedFiles) => {
    let toProcess = acceptedFiles;
    if (preventDuplicates) {
      const found = (acceptedFiles ?? []).filter(
        (file) => files.some((existing) => existing.file.name === file.name)
      );
      if (found.length > 0) {
        setDuplicates(found);
        onDuplicate?.(found);
      }
      toProcess = acceptedFiles.filter(
        (file) => !found.some((dup) => dup.name === file.name)
      );
      if (toProcess.length === 0) return;
    }
    const newFiles = toProcess.map((file) => ({
      file,
      status: onAccept ? types.FileStatus.UPLOADING : types.FileStatus.SUCCESS,
      uid: uuid.v4()
    }));
    try {
      setFiles((prevFiles) => [...prevFiles, ...newFiles]);
      if (onAccept) {
        const updatedFiles = await onAccept(newFiles);
        setFiles((prevFiles) => updateFiles(prevFiles, updatedFiles));
      }
    } catch (e) {
      const updatedFiles = newFiles.map((f) => ({
        ...f,
        status: types.FileStatus.ERROR
      }));
      setFiles((prevFiles) => updateFiles(prevFiles, updatedFiles));
    }
  };
  const addMimeTypes = (key, acc) => {
    types.AcceptSpecificMap[key].forEach((mimeType) => {
      acc[mimeType] = [];
    });
  };
  const mappedAccept = Object.keys(accept).reduce((acc, key) => {
    const mimeTypes = accept[key];
    if (Array.isArray(mimeTypes) && mimeTypes.length === 0) {
      addMimeTypes(key, acc);
    } else if (mimeTypes) {
      mimeTypes.forEach((mimeType) => {
        acc[mimeType] = [];
      });
    }
    return acc;
  }, {});
  const { getRootProps, getInputProps, isDragActive } = reactDropzone.useDropzone({
    onDrop,
    accept: mappedAccept,
    multiple: allowMultiple,
    maxSize,
    onDropRejected: onFileRejected
  });
  const handleRemoveFile = React.useCallback((index) => {
    setFiles((prevFiles) => prevFiles.filter((_, i) => i !== index));
  }, []);
  const handleClearFiles = React.useCallback(() => {
    setFiles([]);
  }, []);
  const isFileLimitReached = !allowMultiple && files.length > 0;
  return {
    onOpen,
    onClose,
    onDelete: handleRemoveFile,
    dropzoneProps: {
      getInputProps,
      getRootProps,
      isDragActive,
      isFileLimitReached
    },
    open: isOpen,
    files,
    duplicates,
    onClearFiles: handleClearFiles
  };
};

exports.useFileUpload = useFileUpload;
